@@ -1,12 +1,14 @@
-from Errors import CompileError, error
-import ExprNodes
-from ExprNodes import IntNode, NameNode, AttributeNode
-import Options
-from Code import UtilityCode, TempitaUtilityCode
-from UtilityCode import CythonUtilityCode
-import Buffer
-import PyrexTypes
-import ModuleNode
+from __future__ import absolute_import
+
+from .Errors import CompileError, error
+from . import ExprNodes
+from .ExprNodes import IntNode, NameNode, AttributeNode
+from . import Options
+from .Code import UtilityCode, TempitaUtilityCode
+from .UtilityCode import CythonUtilityCode
+from . import Buffer
+from . import PyrexTypes
+from . import ModuleNode
 
 START_ERR = "Start must not be given."
 STOP_ERR = "Axis specification only allowed in the 'step' slot."
@@ -75,7 +77,7 @@ def put_init_entry(mv_cname, code):
 
 def mangle_dtype_name(dtype):
     # a dumb wrapper for now; move Buffer.mangle_dtype_name in here later?
-    import Buffer
+    from . import Buffer
     return Buffer.mangle_dtype_name(dtype)
 
 #def axes_to_str(axes):
@@ -180,6 +182,9 @@ def valid_memslice_dtype(dtype, i=0):
     if dtype.is_complex and dtype.real_type.is_int:
         return False
 
+    if dtype is PyrexTypes.c_bint_type:
+        return False
+
     if dtype.is_struct and dtype.kind == 'struct':
         for member in dtype.scope.var_entries:
             if not valid_memslice_dtype(member.type):
@@ -273,7 +278,7 @@ class MemoryViewSliceBufferEntry(Buffer.BufferEntry):
         return bufp
 
     def generate_buffer_slice_code(self, code, indices, dst, have_gil,
-                                   have_slices):
+                                   have_slices, directives):
         """
         Slice a memoryviewslice.
 
@@ -359,6 +364,8 @@ class MemoryViewSliceBufferEntry(Buffer.BufferEntry):
                                      "All preceding dimensions must be "
                                      "indexed and not sliced")
 
+                wraparound = int(directives['wraparound'])
+                boundscheck = int(directives['boundscheck'])
                 d = locals()
                 code.put(load_slice_util("SliceIndex", d))
 
@@ -432,13 +439,16 @@ def get_is_contig_utility(c_contig, ndim):
 
     return utility
 
+
 def copy_src_to_dst_cname():
     return "__pyx_memoryview_copy_contents"
 
+
 def verify_direct_dimensions(node):
     for access, packing in node.type.axes:
         if access != 'direct':
-            error(self.pos, "All dimensions must be direct")
+            error(node.pos, "All dimensions must be direct")
+
 
 def copy_broadcast_memview_src_to_dst(src, dst, code):
     """
@@ -655,7 +665,7 @@ def get_axes_specs(env, axes):
             if entry.name in view_constant_to_access_packing:
                 axes_specs.append(view_constant_to_access_packing[entry.name])
             else:
-                raise CompilerError(axis.step.pos, INVALID_ERR)
+                raise CompileError(axis.step.pos, INVALID_ERR)
 
         else:
             raise CompileError(axis.step.pos, INVALID_ERR)
@@ -723,6 +733,7 @@ def get_axes_specs(env, axes):
 
     return axes_specs
 
+
 def validate_axes(pos, axes):
     if len(axes) >= Options.buffer_max_dims:
         error(pos, "More dimensions than the maximum number"
@@ -731,31 +742,27 @@ def validate_axes(pos, axes):
 
     return True
 
-def all(it):
-    for item in it:
-        if not item:
-            return False
-    return True
 
 def is_cf_contig(specs):
     is_c_contig = is_f_contig = False
 
-    if (len(specs) == 1 and specs == [('direct', 'contig')]):
+    if len(specs) == 1 and specs == [('direct', 'contig')]:
         is_c_contig = True
 
     elif (specs[-1] == ('direct','contig') and
-          all([axis == ('direct','follow') for axis in specs[:-1]])):
+          all(axis == ('direct','follow') for axis in specs[:-1])):
         # c_contiguous: 'follow', 'follow', ..., 'follow', 'contig'
         is_c_contig = True
 
     elif (len(specs) > 1 and
         specs[0] == ('direct','contig') and
-        all([axis == ('direct','follow') for axis in specs[1:]])):
+        all(axis == ('direct','follow') for axis in specs[1:])):
         # f_contiguous: 'contig', 'follow', 'follow', ..., 'follow'
         is_f_contig = True
 
     return is_c_contig, is_f_contig
 
+
 def get_mode(specs):
     is_c_contig, is_f_contig = is_cf_contig(specs)
 
@@ -914,8 +921,7 @@ memviewslice_init_code = load_memview_c_utility(
     context=dict(context, BUF_MAX_NDIMS=Options.buffer_max_dims),
     requires=[memviewslice_declare_code,
               Buffer.acquire_utility_code,
-              atomic_utility,
-              Buffer.typeinfo_compare_code],
+              atomic_utility],
 )
 
 memviewslice_index_helpers = load_memview_c_utility("MemviewSliceIndex")
@@ -948,4 +954,4 @@ view_utility_whitelist = ('array', 'memoryview', 'array_cwrapper',
                           'indirect_contiguous')
 
 memviewslice_declare_code.requires.append(view_utility_code)
-copy_contents_new_utility.requires.append(view_utility_code)
\ No newline at end of file
+copy_contents_new_utility.requires.append(view_utility_code)
