@@ -3,22 +3,23 @@
 #   Cython Scanner
 #
 
-import sys
+from __future__ import absolute_import
+
 import os
 import platform
 
 import cython
-cython.declare(EncodedString=object, string_prefixes=object, raw_prefixes=object, IDENT=unicode,
-               print_function=object)
+cython.declare(EncodedString=object, any_string_prefix=unicode, IDENT=unicode,
+               print_function=object, error=object, warning=object)
 
-from Cython import Plex, Utils
-from Cython.Plex.Scanners import Scanner
-from Cython.Plex.Errors import UnrecognizedInput
-from Errors import CompileError, error
-from Lexicon import string_prefixes, raw_prefixes, make_lexicon, IDENT
-from Future import print_function
+from .. import Utils
+from ..Plex.Scanners import Scanner
+from ..Plex.Errors import UnrecognizedInput
+from .Errors import error, warning
+from .Lexicon import any_string_prefix, make_lexicon, IDENT
+from .Future import print_function
 
-from StringEncoding import EncodedString
+from .StringEncoding import EncodedString
 
 debug_scanner = 0
 trace_scanner = 0
@@ -68,6 +69,9 @@ class CompileTimeScope(object):
     def declare(self, name, value):
         self.entries[name] = value
 
+    def update(self, other):
+        self.entries.update(other)
+
     def lookup_here(self, name):
         return self.entries[name]
 
@@ -94,11 +98,15 @@ def initial_compile_time_env():
         import __builtin__ as builtins
     except ImportError:
         import builtins
+
     names = ('False', 'True',
-        'abs', 'bool', 'chr', 'cmp', 'complex', 'dict', 'divmod', 'enumerate',
-        'float', 'hash', 'hex', 'int', 'len', 'list', 'long', 'map', 'max', 'min',
-        'oct', 'ord', 'pow', 'range', 'reduce', 'repr', 'round', 'slice', 'str',
-        'sum', 'tuple', 'xrange', 'zip')
+             'abs', 'all', 'any', 'ascii', 'bin', 'bool', 'bytearray', 'bytes',
+             'chr', 'cmp', 'complex', 'dict', 'divmod', 'enumerate', 'filter',
+             'float', 'format', 'frozenset', 'hash', 'hex', 'int', 'len',
+             'list', 'long', 'map', 'max', 'min', 'oct', 'ord', 'pow', 'range',
+             'repr', 'reversed', 'round', 'set', 'slice', 'sorted', 'str',
+             'sum', 'tuple', 'xrange', 'zip')
+
     for name in names:
         try:
             benv.declare(name, getattr(builtins, name))
@@ -202,7 +210,18 @@ class FileSourceDescriptor(SourceDescriptor):
         return lines
 
     def get_description(self):
-        return self.path_description
+        try:
+            return os.path.relpath(self.path_description)
+        except ValueError:
+            # path not under current directory => use complete file path
+            return self.path_description
+
+    def get_error_description(self):
+        path = self.filename
+        cwd = Utils.decode_filename(os.getcwd() + os.path.sep)
+        if path.startswith(cwd):
+            return path[len(cwd):]
+        return path
 
     def get_filenametable_entry(self):
         return self.filename
@@ -239,6 +258,8 @@ class StringSourceDescriptor(SourceDescriptor):
     def get_description(self):
         return self.name
 
+    get_error_description = get_description
+
     def get_filenametable_entry(self):
         return "stringsource"
 
@@ -278,14 +299,17 @@ class PyrexScanner(Scanner):
             self.compile_time_env = initial_compile_time_env()
             self.compile_time_eval = 1
             self.compile_time_expr = 0
+            if hasattr(context.options, 'compile_time_env') and \
+               context.options.compile_time_env is not None:
+                self.compile_time_env.update(context.options.compile_time_env)
         self.parse_comments = parse_comments
         self.source_encoding = source_encoding
         if filename.is_python_file():
             self.in_python_file = True
-            self.keywords = cython.set(py_reserved_words)
+            self.keywords = set(py_reserved_words)
         else:
             self.in_python_file = False
-            self.keywords = cython.set(pyx_reserved_words)
+            self.keywords = set(pyx_reserved_words)
         self.trace = trace_scanner
         self.indentation_stack = [0]
         self.indentation_char = None
@@ -322,9 +346,7 @@ class PyrexScanner(Scanner):
     }
 
     def begin_string_action(self, text):
-        if text[:1] in string_prefixes:
-            text = text[1:]
-        if text[:1] in raw_prefixes:
+        while text[:1] in any_string_prefix:
             text = text[1:]
         self.begin(self.string_states[text])
         self.produce('BEGIN_STRING')
@@ -461,7 +483,13 @@ class PyrexScanner(Scanner):
         self.expect('DEDENT',
             "Expected a decrease in indentation level")
 
-    def expect_newline(self, message = "Expected a newline"):
+    def expect_newline(self, message="Expected a newline", ignore_semicolon=False):
         # Expect either a newline or end of file
+        useless_trailing_semicolon = None
+        if ignore_semicolon and self.sy == ';':
+            useless_trailing_semicolon = self.position()
+            self.next()
         if self.sy != 'EOF':
             self.expect('NEWLINE', message)
+        if useless_trailing_semicolon is not None:
+            warning(useless_trailing_semicolon, "useless trailing semicolon")
